@@ -37,54 +37,62 @@ class PromptFormatter:
     def __init__(self):
         self.cache = {}
 
-    def __call__(self, template=DEFAULT_PROMPT, fields=None):
+    def __call__(self, template=DEFAULT_PROMPT, fields=None, **kwargs):
         """Formats a xonsh prompt template string."""
+
+        # keep cache only during building prompt
+        self.cache.clear()
+
         if fields is None:
             self.fields = builtins.__xonsh__.env.get("PROMPT_FIELDS", PROMPT_FIELDS)
         else:
             self.fields = fields
         try:
-            prompt = self._format_prompt(template=template)
+            prompt = self._format_prompt(template=template, **kwargs)
         except Exception:
             return _failover_template_format(template)
-        # keep cache only during building prompt
-        self.cache.clear()
         return prompt
 
-    def _format_prompt(self, template=DEFAULT_PROMPT):
+    def _format_prompt(self, template=DEFAULT_PROMPT, **kwargs):
+        return "".join(self._get_tokens(template, **kwargs))
+
+    def _get_tokens(self, template, **kwargs):
         template = template() if callable(template) else template
         toks = []
         for literal, field, spec, conv in xt.FORMATTER.parse(template):
             toks.append(literal)
-            entry = self._format_field(field, spec, conv)
+            entry = self._format_field(field, spec, conv, idx=len(toks), **kwargs)
             if entry is not None:
                 toks.append(entry)
-        return "".join(toks)
+        return toks
 
-    def _format_field(self, field, spec, conv):
+    def _format_field(self, field, spec, conv, **kwargs):
         if field is None:
             return
         elif field.startswith("$"):
             val = builtins.__xonsh__.env[field[1:]]
             return _format_value(val, spec, conv)
         elif field in self.fields:
-            val = self._get_field_value(field)
+            val = self._get_field_value(field, spec=spec, conv=conv, **kwargs)
             return _format_value(val, spec, conv)
         else:
             # color or unknown field, return as is
             return "{" + field + "}"
 
-    def _get_field_value(self, field):
+    def _get_field_value(self, field, **kwargs):
         field_value = self.fields[field]
         if field_value in self.cache:
             return self.cache[field_value]
+        return self._no_cache_field_value(field, field_value, **kwargs)
+
+    def _no_cache_field_value(self, field, field_value, **_):
         try:
             value = field_value() if callable(field_value) else field_value
             self.cache[field_value] = value
         except Exception:
             print("prompt: error: on field {!r}" "".format(field), file=sys.stderr)
             xt.print_exception()
-            value = "{{BACKGROUND_RED}}{{ERROR:{}}}{{NO_COLOR}}".format(field)
+            value = "{{BACKGROUND_RED}}{{ERROR:{}}}{{RESET}}".format(field)
         return value
 
 
@@ -118,21 +126,21 @@ def default_prompt():
         dp = (
             "{env_name}"
             "{BOLD_GREEN}{user}@{hostname}"
-            "{BOLD_BLUE} {cwd} {prompt_end}{NO_COLOR} "
+            "{BOLD_BLUE} {cwd} {prompt_end}{RESET} "
         )
     elif xp.ON_WINDOWS and not xp.win_ansi_support():
         dp = (
             "{env_name}"
             "{BOLD_INTENSE_GREEN}{user}@{hostname}{BOLD_INTENSE_CYAN} "
-            "{cwd}{branch_color}{curr_branch: {}}{NO_COLOR} "
-            "{BOLD_INTENSE_CYAN}{prompt_end}{NO_COLOR} "
+            "{cwd}{branch_color}{curr_branch: {}}{RESET} "
+            "{BOLD_INTENSE_CYAN}{prompt_end}{RESET} "
         )
     else:
         dp = (
             "{env_name}"
             "{BOLD_GREEN}{user}@{hostname}{BOLD_BLUE} "
-            "{cwd}{branch_color}{curr_branch: {}}{NO_COLOR} "
-            "{BOLD_BLUE}{prompt_end}{NO_COLOR} "
+            "{cwd}{branch_color}{curr_branch: {}}{RESET} "
+            "{BOLD_BLUE}{prompt_end}{RESET} "
         )
     return dp
 
@@ -180,7 +188,7 @@ def multiline_prompt(curr=""):
             basetoks.append(("\001" + pre + "\002", post))
             baselen += len(post)
     if baselen == 0:
-        return xt.format_color("{NO_COLOR}" + tail, hide=True)
+        return xt.format_color("{RESET}" + tail, hide=True)
     toks = basetoks * (headlen // baselen)
     n = headlen % baselen
     count = 0
@@ -196,7 +204,7 @@ def multiline_prompt(curr=""):
         count = newcount
         if n <= count:
             break
-    toks.append((xt.format_color("{NO_COLOR}", hide=True), tail))
+    toks.append((xt.format_color("{RESET}", hide=True), tail))
     rtn = "".join(itertools.chain.from_iterable(toks))
     return rtn
 
